# -*- coding: utf-8 -*-

import datetime
import urllib
import urllib2
import re
import os
import pytz

from goscale import conf
#from goscale import decorators
#from dynamicsettings import settings
#from goscale import feeds

from django.core.cache import cache #from google.appengine.api import memcache as cache

from django import http
from django.core import urlresolvers
from django.utils import html as html_utils
from django.utils import simplejson
from django.utils import translation
from django import template
from django.utils import importlib
from django.core import exceptions
from django.contrib.sites import models as site_models
from django.conf import settings
#
#
#def get_site(request):
#    if site_models.Site._meta.installed:
#        site = site_models.Site.objects.get_current()
#    else:
#        site = site_models.RequestSite(request)
#    return site
#
#def get_source_setting(source_name, setting_name):
#    return settings.GOSCALE_CONTENT_TYPES[source_name][setting_name]
#
#def get_template_setting(template_name, setting_name=None):
#    template = settings.GOSCALE_TEMPLATES[template_name]
#    if setting_name:
#        return template[setting_name]
#    return template
#
#def get_module_setting(module_name, setting_name):
#    return settings.GOSCALE_MODULES[module_name][setting_name]
#
##@decorators.cache_func
#def get_page_by_id(id):
#    try:
#        return models.Page.objects.get(id=id)
#    except models.Page.DoesNotExist:
#        return None
#
##@decorators.cache_func
#def get_all_pages():
#    ''' Caches DB request to models.Page.all().order('sort_id') '''
#    return list(models.Page.objects.all().order_by('sort_id'))
#
##@decorators.cache_func
#def get_page_by_absolute_url(url):
#    ''' Returns page object or raises raises HTTP404 if page not found'''
#    all_pages = get_all_pages()
#    if url=='/' or not url:
#        for page in all_pages:
#            if page.sort_id == 0:
#                return page
#    for page in all_pages:
#        if page.absolute_url == url:
#            return page
#    return None
#
##@decorators.cache_func
#def get_temporary_page(absolute_url, slug=None):
#    page_title = ''
#    if slug is not None:
#        try:
#            post = models.Post.objects.get(slug=slug)
#            page_title = post.title
#        except models.Post.DoesNotExist:
#            pass
#    page_titles = {}
#    for language_definition in settings.LANGUAGES:
#        page_titles[language_definition[0]] = page_title
#    page_attributes = {'title': page_titles}
#    return models.Page(
#        parent_page=None,
#        modules=None,
#        attributes=simplejson.dumps(page_attributes),
#        slug=slug,
#        template=None,
#        sort_id=1000,
#        visible=True,
#        login_required=False,
#        absolute_url=absolute_url,
#        languages=','.join([language_definition[0] for language_definition in settings.LANGUAGES]),
#    )
#
#def get_ajax_post(request, slug, format='html'):
#    posts_dir = settings.GOSCALE_POSTS_TEMPLATE_DIR
#    post = get_post_by_slug(slug)
#    post_type = post.content_type
#    post_json = post.json()
#    if format=='json':
#        ajax_content = post_json
#    elif format=='rss' or format=='atom':
#        self_link = 'http://%s%s' % (request.get_host(), request.get_full_path())
#        feed = feeds.XMLFeed(type=format,
#                          data=[post_json],
#                          title=post_json['title'] if post_json.get('title') is not None else '',
#                          description='',
#                          link=self_link)
#        ajax_content = feed.get_feed(object(), request).writeString(encoding='UTF-8')
#    else:
#        content_dict = {
#                    'data': post.json(),
#                   }
#        context_dict = dict(content=content_dict, request=request)
#        get_template = request.GET.get('template', None)
#        if get_template:
#            template_file = os.path.join(posts_dir, get_template)
#        else:
#            template_file = os.path.join(posts_dir, '%s.html' % post_type)
#        ajax_content = template.loader.render_to_string(template_file, context_dict)
#    return ajax_content
#
##@decorators.cache_func
#def get_breadcrumb(current_page, lang, def_lang, user):
#    breadcrumb_obj = []
#    while True:
#        breadcrumb_obj.append(single_page_menu_obj(current_page, lang, def_lang, user))
#        if not current_page.parent_page:
#            break
#        current_page = get_page_by_id(current_page.parent_page.pk)
#    breadcrumb_obj.reverse()
#    return breadcrumb_obj
#
##@decorators.cache_func
#def make_menu_obj(lang, def_lang, user, current_page=None):
#    """ Creates an object used to display menu. Object structure:
#    top level :  | * | * |-*-| * |
#    second lev:  |  *  |-*-|  *  |
#    current lev: |-*-| * | * | * | <--active page/parent nodes have attribute ['selected'] set
#    cur+1 lev:   etc.
#    ...
#    single node struct defined in single_page_menu_obj()
#
#    """
#    all_pages = get_all_pages()
#    menu_obj = []
#    children = []
#    """collect children - bottom level"""
#    for page in all_pages:
#        if current_page and page.parent_page and page.parent_page.id == current_page.id:
#            child = single_page_menu_obj(page, lang, def_lang, user)
#            if child:
#                children.append(child)
#    if children:
#        menu_obj.append(children)
#    """ collect siblings, parents and parent's siblings """
#    selected_page = current_page
#    while True:
#        sibling_pages = get_page_siblings(selected_page);
#        siblings = []
#        for sibling_page in sibling_pages:
#            sibling = single_page_menu_obj(sibling_page, lang, def_lang, user)
#            if not sibling:
#                continue
#            if selected_page and sibling_page.id == selected_page.id:
#                sibling['selected'] = True
#            siblings.append(sibling)
#        menu_obj.append(siblings)
#        if not selected_page:
#            break
#        if not selected_page.parent_page:
#            break
#        selected_page = get_page_by_id(selected_page.parent_page.id)
#    menu_obj.reverse()
#    return menu_obj
#
##@decorators.cache_func
#def single_page_menu_obj(page, lang, def_lang, user):
#    if page.visible is False or not lang in page.languages\
#    or (settings.GOSCALE_SHOW_PROTECTED_PAGES is False and page.login_required is True\
#        and user.is_anonymous() is True):
#        return None
#    ret_obj = {
#        'slug': page.slug,
#        'title': get_page_title(page, lang),
#        'url': page.absolute_url,
#    }
#    return ret_obj
#
##@decorators.cache_func
#def get_page_siblings(current_page):
#    all_pages = get_all_pages()
#    siblings = []
#    current_page_parent_key = None
#    if current_page and current_page.parent_page:
#        current_page_parent_key = current_page.parent_page.pk
#    for page in all_pages:
#        page_parent_key = None
#        if page.parent_page:
#            page_parent_key = page.parent_page.pk
#        if page_parent_key == current_page_parent_key:
#            siblings.append(page)
#    return siblings
#
#def get_page_title(page, lang):
#    return get_attribute_by_language(page, 'title', lang)
#
##@decorators.cache_func
#def get_language_from_path(url_path):
#    """
#    Returns language and url_path without language_prefix from the url
#    """
#    for language_definition in settings.LANGUAGES:
#        if url_path.startswith('/%s/' % language_definition[0]):
#            return language_definition[0]
#        if url_path=='/':
#            return conf.DEFAULT_LANGUAGE
#    return translation.get_language()
#
##@decorators.cache_func
#def get_absolute_url(url_path, lang):
#    if url_path.startswith('/%s/' % lang):
#        return url_path.replace('/%s' % lang, '', 1)
#    return url_path
#
##@decorators.cache_func
#def get_all_languages():
#    all_lang = []
#    for language_definition in settings.LANGUAGES:
#        all_lang.append([language_definition[0], language_definition[1]])
#    all_lang.sort(key=lambda language_tuple: language_tuple[1])
#    return all_lang
#
##@decorators.cache_func
#def get_languages(url_path):
#    return get_language_from_path(url_path), get_all_languages()
#
##@decorators.cache_func
#def get_language_redirect(url_path, lang, is_page=True):
#    user_lang = translation.get_language()
#    if user_lang != lang:
#        set_language_url = urlresolvers.reverse('set_language')
#        lang_prefix = get_language_prefix(lang, user_lang)
#        if user_lang == conf.DEFAULT_LANGUAGE:
#            url_path = url_path.replace(lang_prefix,'')
#        else:
#            url_path = url_path.replace(lang_prefix, '/%s' % user_lang)
#        redirect_url = '%s?%s=%s&language=%s' % (set_language_url,
#                                                 settings.REDIRECT_FIELD_NAME,
#                                                 url_path, user_lang)
#        if is_page is True:
#            redirect_url = '%s&is_page=1' % redirect_url
#        return redirect_url
#    return None
#
##@decorators.cache_func
#def get_language_prefix(lang, def_lang):
#    if lang==def_lang:
#        return ''
#    else:
#        return '/%s' % lang
#
def process_feed_tz_delta(date_string):
    date_match = re.findall('[\-\+]\d{2}:?\d{2}', date_string)
    if date_match:
        factor = 1
        tz_string = date_match[0].replace(':', '')
        if tz_string.startswith('-'):
            factor = -1
        tz_delta = ((int(tz_string[-4:-2]) * 3600) + (int(tz_string[-2:]) * 60)) * factor
        return tz_delta
    return 0

def get_datetime_now():
    """ Returns now() in UTC timezone

    More info: http://pytz.sourceforge.net/
    """
    return datetime.datetime.now(tz=pytz.utc)

def get_utc(dt):
    """ Converts datetime to UTC timezone

    More info: http://pytz.sourceforge.net/
    """
    return dt.replace(tzinfo=pytz.utc)

def get_datetime_by_parsed(pDate, tz_delta=0):
    if not pDate:
        return None
    dt = datetime.datetime(pDate[0], pDate[1],pDate[2], pDate[3], pDate[4], pDate[5]) + datetime.timedelta(seconds=tz_delta)
    return get_utc(dt)

##@decorators.cache_func
#def get_params(data_source_dict, data_source_obj):
#    """Intersect a data_source dict and a data_source_obj and returns a dict
#    """
#    ret_dict = data_source_dict
#    try:
#        ret_dict.update(simplejson.loads(data_source_obj.attributes))
#    except ValueError:
#        pass
#    ret_dict.update({
#                     'source_id': data_source_obj.source_id,
#                     'type': data_source_obj.type,
#                    })
#    return ret_dict
#
#def get_params_from_request(request, exclude=None):
#    """Returns additional datasource params from request
#    """
#    if exclude is None:
#        exclude = []
#    ret_dict = {}
#    for key, value in request.GET.iteritems():
#        if key not in exclude:
#            ret_dict[key] = value
#    return ret_dict

def get_short_summary(html):
    max_length_shortcontent = conf.GOSCALE_POST_SUMMARY_LIMIT
    content_temp = html_utils.strip_tags(html)
    if len(content_temp) < max_length_shortcontent:
        return content_temp
    shortcontent_temp = content_temp[:max_length_shortcontent]
    if shortcontent_temp.rfind('.') != -1:
        shortcontent_temp = shortcontent_temp[:shortcontent_temp.rfind('.')+1]
    else:
        shortcontent_temp = shortcontent_temp[:shortcontent_temp.rfind(' ')+1] + '...'
    return shortcontent_temp

##@decorators.cache_func
#def filter_content_types(content_type):
#    return [class_name for class_name in settings.GOSCALE_CONTENT_TYPES if settings.CONTENT_TYPES['class_name'].get('type') == content_type]
#
##@decorators.cache_func
#def get_source_by_url(url):
#    try:
#        return models.DataSource.objects.get(url=url)
#    except models.DataSource.DoesNotExist:
#        raise #for now
#        return None
#
##@decorators.cache_func
#def get_source_by_id(source_id):
#    try:
#        return models.DataSource.objects.get(source_id=source_id)
#    except models.DataSource.DoesNotExist:
#        raise #for now
#        return None
#
##@decorators.cache_func
#def get_attribute_by_language(page, attr_name, lang):
#    try:
#        attributes = simplejson.loads(page.attributes)
#    except ValueError:
#        #raise ValueError('Attribute "%s" for page "%s" and language "%s" could not be decoded.' % (attr_name, page, lang))
#        attributes = {}
#    attribute = attributes.get(attr_name)
#    if attribute:
#        return attribute.get(lang) or attribute.get(conf.DEFAULT_LANGUAGE) or attribute[attribute.keys()[0]] or None
#    return None
#
##@decorators.cache_func
#def get_all_modules():
#    return list(models.Module.objects.all())
#
##@decorators.cache_func
#def get_module_by_module_id(module_id):
#    try:
#        return models.Module.objects.get(module_id=module_id)
#    except models.Module.DoesNotExist:
#        raise #for now
#        return None
#
##@decorators.cache_func
#def get_modules_by_module_ids(module_ids):
#    #list() is evaluating the queryset, so the results are cached, not the queryset
#    return list(models.Module.objects.filter(module_id__in=module_ids))
#
##@decorators.cache_func
#def get_modules_for_page(page, language=conf.DEFAULT_LANGUAGE):
#    """Get the module objects for a given page,language
#    """
#    try:
#        page_modules = simplejson.loads(page.modules)
#    except ValueError:
#        page_modules = []
#    modules = []
#    for page_module in page_modules:
#        module_dict = {}
#        module = get_module_by_module_id(page_module['id'])
#        module_dict.update(page_module)
#        #try:
#        #    module_json = simplejson.loads(module.data_sources)
#        #except ValueError:
#        #    #raise ('Data sources for module "%s" on page "%s" and language "%s" could not be decoded.' % (module, page, lang))
#        #    module_json = {}
#        #if language in module_json:
#        module_dict.update(settings.GOSCALE_MODULES[module.type])
#        module_dict['title'] = simplejson.loads(module.title).get(language, "")
#        module_dict['attributes'] = simplejson.loads(module.attributes).get(language, None)
#        modules.append(module_dict)
#    return modules
#
##@decorators.cache_func
#def get_module_class(module_type):
#    module_name_parts = module_type.split('_')
#    return '%sModule' % (''.join([part.title().replace(' ','') for part in module_name_parts]))
#
##@decorators.cache_func
#def get_modules_for_panel(modules, panel):
#    modules_to_render = []
#    module_number = 0 #the module number (index) in the given panel
#    for module in modules:
#        try:
#            module_panel = module['panel']
#        except TypeError:
#            module_panel = module.panel
#        if panel==module_panel:
#            #add container and class to module
#            module['container'] = '%s-module%s' % (panel.replace(' ','_'), module_number)
#            module['class'] = get_module_class(module['type'])
#            modules_to_render.append(module)
#            module_number += 1
#    return modules_to_render
#
##@decorators.cache_func
#def get_post_by_slug(slug):
#    try:
#        return models.Post.objects.get(slug=slug)
#    except models.Module.DoesNotExist:
#        raise #for now
#        return None
#
#def get_unique_panels(page):
#    try:
#        page_panels = get_template_setting(page.template, 'panels')
#    except KeyError:
#        page_panels = []
#    for panel in settings.GOSCALE_COMMON_PANELS:
#        if panel not in page_panels:
#            page_panels.append(panel)
#    return page_panels
#
#def create_task(params):
#    try:
#        from google.appengine.api.labs import taskqueue
#        return taskqueue.add(url=urlresolvers.reverse('task_handler'), params=params, method='GET')
#    except ImportError:
#        pass #additional code for django only version later
#
##@decorators.cache_func
#def unpack_hash(hash_string):
#    if hash_string.startswith('#!'):
#        hash_string = hash_string.replace('#!', '')
#    hash_module_dict = {}
#    hash_modules = hash_string.split('|')
#    for hash_module in hash_modules:
#        module_hash_name = hash_module[0:hash_module.find('=')]
#        module_name = module_hash_name[hash_module.find('_')+1:]
#        module_hash_params = hash_module.replace('%s=' % module_hash_name, '')
#        module_params = {}
#        for hash_param in module_hash_params.split(';'):
#            param_module = hash_param.split('=')
#            if len(param_module)==1:
#                key = value = hash_param
#            elif len(param_module)==2:
#                key, value = hash_param.split('=')
#            else:
#                raise ValueError('Wrong hash formatting. To many  values separated by ``=`` for module parameters.')
#            module_params[key] = value
#        hash_module_dict[module_name] = module_params
#    return hash_module_dict
#
##@decorators.cache_func
#def pack_hash(hash_dict):
#    hash_list = []
#    for module_key in hash_dict.keys():
#        params_module_list = []
#        for param_key in hash_dict[module_key].keys():
#            param_value = hash_dict[module_key][param_key]
#            if param_key==param_value:
#                params_module_list.append(param_key)
#            else:
#                params_module_list.append('%s=%s' % (param_key, param_value))
#        hash_list.append('mod_%s=%s' % (module_key, ';'.join(params_module_list)))
#    return '#!%s' % '|'.join(hash_list)
#
#def modify_ajax_links(request, posts):
#    """Call this function only when you need to make content crawlable
#    """
#    url_path = request.path_info
#    get_dict = request.GET.copy()
#    if get_dict.get('_escaped_fragment_') is not None:
#        del get_dict['_escaped_fragment_']
#    full_url = '%s%s'
#    if len(get_dict)>0:
#        full_url = '%s?%s'
#    full_url = full_url % (url_path, '&'.join(['%s=%s' % (key, value) for key, value in get_dict.items()]))
#    for i, post in enumerate(posts):
#        posts[i]['ajax_link'] = '%s%s' % (full_url, post['ajax_link'])
#    return posts
#
#
##@decorators.cache_func
#def get_ajax_url(content_type, page, lang, data_source_index, output=None, offset=None, limit=None):
#    ajax_params = {
#        'type': content_type,
#        'page': page.absolute_url,
#        'language': lang,
#        'source': data_source_index,
#    }
#    if output:
#        ajax_params['output'] = output
#    if offset:
#        ajax_params['offset'] = offset
#    if limit:
#        ajax_params['limit'] = limit
#
#    join_at = '?' if ajax_params else ''
#    ajax_url = join_at.join([
#                '%s' % (urlresolvers.reverse('ajax')),
#                '&'.join(['%s=%s' % (param, ajax_params[param]) for param in ajax_params])
#    ])
#    return ajax_url
#
##@decorators.cache_func
#def get_data_sources_for_ids(source_ids):
#    return list(models.DataSource.objects.filter(source_id__in=source_ids))
#
##@decorators.cache_func
#def get_posts_for_data_sources(data_sources, **kwargs):
#    posts = models.Post.objects.filter(data_source__in=data_sources)
#    if 'order_by' in kwargs:
#        posts = posts.order_by(kwargs['order_by'])
#    if 'offset' in kwargs and 'limit' in kwargs:
#        return posts[kwargs['offset']:kwargs['limit']]
#    return list(posts)
#
#def read_cookie(request, cookie_name):
#    return request.COOKIES.get(cookie_name)
#
#def read_tz_delta_cookie(request):
#    tz_delta = read_cookie(request, 'user_tz_delta')
#    if tz_delta is None:
#        return settings.GOSCALE_STANDARD_TZ_DELTA*60
#    return int(tz_delta)
#
#def get_request_from_context(context):
#    request = context.get('request')
#    if not request:
#        raise template.TemplateSyntaxError('No template variable "request" in the context. Please add the ``django.core.context_processors.request`` context processors to your settings.TEMPLATE_CONTEXT_PROCESSORS set')
#    return request
#
def dict2obj(d):
    """A helper function which convert a dict to an object.
    """
    if isinstance(d, (list, tuple)):
        d = [dict2obj(x) for x in d]
    if not isinstance(d, dict):
        return d
    class ObjectFromDict(object):
        pass
    o = ObjectFromDict() #an object created from a dict
    for k in d:
        o.__dict__[k] = dict2obj(d[k])
    return o
#
def forge_request(url):
    """A helper function which forges a request for fetching json from vimeo and youku.
    """
    return urllib2.Request(url, None, { 'User-Agent' : 'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT)' })
#
#
#def get_class_for_path(path):
#    i = path.rfind('.')
#    module, attr = path[:i], path[i+1:]
#    try:
#        mod = importlib.import_module(module)
#    except ImportError, e:
#        raise exceptions.ImproperlyConfigured('Error loading class %s: "%s"' % (module, e))
#    try:
#        module_class = getattr(mod, attr)
#    except AttributeError:
#        raise exceptions.ImproperlyConfigured('Module "%s" does not define a class named "%s"' % (module, attr))
#    return module_class
#
#def queue2tree(comments_queue, comments_tree):
#    while len(comments_queue)!=0:
#        comment = comments_queue.popleft()
#        if not recursively_parse_comment_tree(comment, comments_tree):
#            comments_queue.append(comment)
#
#
#def recursively_parse_comment_tree(comment, nested_comment_tree):
#    # if the comment does not have a parent comment, then append the node directly to the tree.
#    if not comment.parent_comment:
#        nested_comment_tree.append({
#            'comment': comment.dict(),
#            'children': []
#        })
#        return True
#    # else the comment has a parent.
#    for comment_node in nested_comment_tree:
#        # go through every node in the tree, if the comment's parent is there, then append the noce to the children of such commet.
#        # if not, check the children of this comment.
#        # whenever found one, this function will return True.
#        # if a true is cought inside of the for loop, exit the loop and return directly.
#        if comment.parent_comment.id == comment_node['comment']['id']:
#            comment_node['children'].append({
#                'comment': comment.dict(),
#                'children': []
#            })
#            return True
#        if recursively_parse_comment_tree(comment, comment_node['children']):
#            return True
#    return False
#
#def comment_is_prepend():
#    default_ordering = settings.GOSCALE_DEFAULT_COMMENTS_SORTING_ORDER
#    if default_ordering[:1] == '-':
#        return 'true'
#    return 'false'
#
#def nested_remove_comment(comment):
#    parent_comment = comment.parent_comment
#    if not parent_comment:
#        return remove_comment(comment)
#    if not parent_comment.is_removed:
#        return remove_comment(comment)
#    parent_sublings = models.Comment.objects.filter(parent_comment = parent_comment)
#    if len(parent_sublings)!=1:
#        # in this case, there are more children in this parent_comment, then just delete the comment.
#        return remove_comment(comment)
#    else:
#        # in this case, such parent has only one child and this child is being removed.
#        # check nested_removed_comment(parent_comment)
#        id_to_remove = nested_remove_comment(parent_comment)
#    return str(id_to_remove)
#
#def remove_comment(comment):
#    id_to_remove = comment.id
#    comment.delete()
#    return str(id_to_remove)
#
#def add_label_filter(query, label):
#    if label:
#        label = '[%s]' % label
#        query = query.filter(categories__contains=label)
#    return query
#
#def weighted_rating(rating, votes, minimum_votes=settings.GOSCALE_RATING_MINIMUM_VOTES, mean_value=settings.GOSCALE_RATING_MEAN_VALUE):
#    """Bayesian estimate
#    rating- average for the picture
#    votes - number of votes for the picture
#    m - minimum votes required to be listed
#    mean_value - the mean vote across the whole report
#    """
#    if votes==0:
#        return 0.0
#    return (float(votes) / (float(votes) + float(minimum_votes))) * float(rating) + (float(minimum_votes) / (float(votes) + float(minimum_votes))) * float(mean_value)
#
#def send_comment_replied_email(request, parent_comment, comment):
#    site = get_site(request)
#    user = parent_comment.user
#    content_dict = {
#        'site': site,
#        'username': user.username,
#        'parent_comment': parent_comment.comment,
#        'comment': comment.comment,
#        'reply_from': comment.user.username,
#        'path': parent_comment.get_content_object_url(),
#    }
#    comment_replied_subj_template = os.path.join(settings.GOSCALE_ROOT_TEMPLATE_DIR, 'comments', 'email', 'email_comment_replied_subject.txt')
#    subject = template.loader.render_to_string(comment_replied_subj_template, content_dict)
#    # Email subject *must not* contain newlines
#    subject = ''.join(subject.splitlines())
#    comment_replied_msg_template = os.path.join(settings.GOSCALE_ROOT_TEMPLATE_DIR, 'comments', 'email', 'email_comment_replied.txt')
#    message = template.loader.render_to_string(comment_replied_msg_template, content_dict)
#    user.email_user(subject, message, settings.DEFAULT_FROM_EMAIL)
#
#def send_comment_deleted_email(request, comment):
#    site = get_site(request)
#    user = comment.user
#    content_dict = {
#        'site': site,
#        'username': user.username,
#        'comment': comment.comment,
#        'path': comment.get_content_object_url(),
#    }
#    comment_deleted_subj_template = os.path.join(settings.GOSCALE_ROOT_TEMPLATE_DIR, 'comments', 'email', 'email_comment_deleted_subject.txt')
#    subject = template.loader.render_to_string(comment_deleted_subj_template, content_dict)
#    # Email subject *must not* contain newlines
#    subject = ''.join(subject.splitlines())
#    comment_deleted_msg_template = os.path.join(settings.GOSCALE_ROOT_TEMPLATE_DIR, 'comments', 'email', 'email_comment_deleted.txt')
#    message = template.loader.render_to_string(comment_deleted_msg_template, content_dict)
#    user.email_user(subject, message, settings.DEFAULT_FROM_EMAIL)
#
#def get_redirect_value(request):
#    redirect_field_name = settings.REDIRECT_FIELD_NAME
#    if redirect_field_name in request.session:
#        redirect_to = request.session[redirect_field_name]
#        del request.session[redirect_field_name]
#    elif redirect_field_name in request.COOKIES:
#        redirect_to = urllib2.unquote(request.COOKIES[redirect_field_name])
#    elif redirect_field_name in request.GET:
#        redirect_to = request.GET.get(redirect_field_name)
#    elif redirect_field_name in request.POST:
#        redirect_to = request.POST.get(redirect_field_name)
#    else:
#        redirect_to = settings.get('LOGIN_REDIRECT_URL', '/')
#    if not redirect_to or ' ' in redirect_to:
#        redirect_to = settings.get('LOGIN_REDIRECT_URL', '/')
#    elif '//' in redirect_to and re.match(r'[^\?]*//', redirect_to):
#        redirect_to = settings.get('LOGIN_REDIRECT_URL', '/')
#    return redirect_to


from goscale.cms_plugins import GoscaleCMSPluginBase
from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
import models

GOSCALE_FORMS_PLUGIN_TEMPLATES = getattr(settings, 'GOSCALE_FORMS_PLUGIN_TEMPLATES', (
    ('form.html', _('Form')),
    ('form_popup.html', _('Form in a lightbox')),
)) + getattr(settings, 'GOSCALE_FORMS_CUSTOM_PLUGIN_TEMPLATES', ())


class FormPlugin(GoscaleCMSPluginBase):
    """
    Feed plugin for GoScale
    """
    model = models.Form
    name = _("Google Form")
    plugin_templates = GOSCALE_FORMS_PLUGIN_TEMPLATES
    render_template = GOSCALE_FORMS_PLUGIN_TEMPLATES[0][0]
    fieldsets = [
        [_('Form options'), {
            'fields': ['url']
        }]
    ]

plugin_pool.register_plugin(FormPlugin)


# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Vimeo.playlist'
        db.alter_column('cmsplugin_vimeo', 'playlist', self.gf('django.db.models.fields.CharField')(max_length=250, null=True))

        # Changing field 'YouKu.playlist'
        db.alter_column('cmsplugin_youku', 'playlist', self.gf('django.db.models.fields.CharField')(max_length=250, null=True))

        # Changing field 'Youtube.playlist'
        db.alter_column('cmsplugin_youtube', 'playlist', self.gf('django.db.models.fields.CharField')(max_length=250, null=True))

    def backwards(self, orm):

        # User chose to not deal with backwards NULL issues for 'Vimeo.playlist'
        raise RuntimeError("Cannot reverse this migration. 'Vimeo.playlist' and its values cannot be restored.")

        # User chose to not deal with backwards NULL issues for 'YouKu.playlist'
        raise RuntimeError("Cannot reverse this migration. 'YouKu.playlist' and its values cannot be restored.")

        # User chose to not deal with backwards NULL issues for 'Youtube.playlist'
        raise RuntimeError("Cannot reverse this migration. 'Youtube.playlist' and its values cannot be restored.")

    models = {
        'cms.cmsplugin': {
            'Meta': {'object_name': 'CMSPlugin'},
            'changed_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2013, 1, 28, 0, 0)'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'level': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'lft': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.CMSPlugin']", 'null': 'True', 'blank': 'True'}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['cms.Placeholder']", 'null': 'True'}),
            'plugin_type': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'}),
            'position': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'rght': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'}),
            'tree_id': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        },
        'goscale.post': {
            'Meta': {'object_name': 'Post'},
            'attributes': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'author': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'categories': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'content_type': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'link': ('django.db.models.fields.URLField', [], {'db_index': 'True', 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'permalink': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'published': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'slug': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'summary': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'})
        },
        'videos.vimeo': {
            'Meta': {'object_name': 'Vimeo', 'db_table': "'cmsplugin_vimeo'"},
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'lightbox': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'playlist': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'posts': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['goscale.Post']", 'symmetrical': 'False'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'})
        },
        'videos.youku': {
            'Meta': {'object_name': 'YouKu', 'db_table': "'cmsplugin_youku'"},
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'lightbox': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'playlist': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'posts': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['goscale.Post']", 'symmetrical': 'False'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'})
        },
        'videos.youtube': {
            'Meta': {'object_name': 'Youtube', 'db_table': "'cmsplugin_youtube'"},
            'channel': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'cmsplugin_ptr': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['cms.CMSPlugin']", 'unique': 'True', 'primary_key': 'True'}),
            'lightbox': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'playlist': ('django.db.models.fields.CharField', [], {'max_length': '250', 'null': 'True', 'blank': 'True'}),
            'posts': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['goscale.Post']", 'symmetrical': 'False'}),
            'template': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['videos']


from goscale.cms_plugins import GoscaleCMSPluginBase
from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
import models

GOSCALE_CALENDAR_PLUGIN_TEMPLATES = getattr(settings, 'GOSCALE_CALENDAR_PLUGIN_TEMPLATES', (
    ('events.html', _('Events list')),
    ('events_mini.html', _('Events mini list (sidebar)')),
    ('datepicker.html', _('Date picker widget')),
)) + getattr(settings, 'GOSCALE_CALENDAR_CUSTOM_PLUGIN_TEMPLATES', ())


class CalendarPlugin(GoscaleCMSPluginBase):
    """
    Google Calendar plugin for GoScale
    """
    model = models.Calendar
    name = _("Calendar")
    plugin_templates = GOSCALE_CALENDAR_PLUGIN_TEMPLATES
    render_template = GOSCALE_CALENDAR_PLUGIN_TEMPLATES[0][0]
    plugin_post_template = 'event.html'
    fieldsets = [
        [_('Calendar options'), {
            'fields': ['url', 'page_size', 'show_past']
        }]
    ]

    def _extra_context(self, context, instance):
        if instance.show_past:
            return {'past_events': instance.get_past_events()}
        return super(CalendarPlugin, self)._extra_context(context, instance)

plugin_pool.register_plugin(CalendarPlugin)


import urllib
import re
import feedparser
import urllib2
import simplejson
import datetime
from goscale import models as goscale_models
from goscale import utils
from goscale import conf
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext as _


class FeedBase(goscale_models.GoscaleCMSPlugin):
    """
    Feed Base Class
    """
    url = models.URLField(max_length=250, verbose_name=_('Feed URL'),
        help_text='ex: http://feeds.bbci.co.uk/news/technology/rss.xml')
    page_size = models.PositiveSmallIntegerField(default=conf.GOSCALE_DEFAULT_PAGE_SIZE,
        verbose_name=_('Posts per page'), help_text=_('set 0 for unlimited.'))
    show_date = models.BooleanField(default=False, verbose_name=_('Show date in posts'),
        help_text=_('If checked the date will be shown along with the post content.'))
    external_links = models.BooleanField(default=False, verbose_name=_('Open external links'),
        help_text=_('If checked posts will link to the original source, otherwise will open internally.'))
    disqus = models.CharField(max_length=50, null=True, blank=True, verbose_name=_('DICQUS shortname'),
        help_text=_('Use it if you want to enable disqus.com comments.'))

    class Meta:
        abstract = True

    def _get_data(self):
        url = self._get_data_source_url()
        if not url:
            return []
        res = urllib2.urlopen(url.encode('utf-8'))
        #TBD get hash and compare to cache value
        feed = feedparser.parse(res.read())
        return feed.entries

    def _get_title(self, entry):
        """ Returns a title for an entry
        """
        return entry.get('title')

    def _store_post(self, stored_entry, entry):
        stored_entry.content_type = 'text/html'

        updated_tz_delta = 0
        updated_string = entry.get('updated') or entry.get('published')
        updated_tz_delta = utils.process_feed_tz_delta(updated_string)

        published_tz_delta = 0
        published_string = entry.get('published') or entry.get('updated')
        published_tz_delta = utils.process_feed_tz_delta(published_string)

        stored_entry.updated = utils.get_datetime_by_parsed(entry.get('updated_parsed') or entry.get('published_parsed'), updated_tz_delta)
        stored_entry.published = utils.get_datetime_by_parsed(entry.get('published_parsed') or entry.get('updated_parsed'), published_tz_delta)
        stored_entry.title = self._get_title(entry)
        content = entry.get('summary')
        if not content:
            content_list = entry.get('content')
            for content_item in content_list:
                content = content_item.get('value') or content_item
        stored_entry.description = content
        stored_entry.summary = utils.get_short_summary(content)
        stored_entry.author = entry.get('author')
        if entry.get('tags'):
            stored_entry.categories = ','.join([tag.get('term') or tag.get('label') for tag in entry.get('tags') if tag.get('term') or tag.get('label')])
            #other stuff we might need
        author_details = entry.get('author_detail')
        if author_details:
            stored_entry.attributes = simplejson.dumps(dict(author_details=author_details))
            #stored_entry.author
        return super(FeedBase, self)._store_post(stored_entry)


class Feed(FeedBase):
    """
    RSS Feed posts
    """
    pass

signals.post_save.connect(goscale_models.update_posts, sender=Feed)


class BlogBase(FeedBase):
    """
    Base for blog posts
    """
    label = models.CharField(max_length=50, blank=True, null=True, verbose_name=_('Label/Tag'),
        help_text='ex: features')

    class Meta:
        abstract = True


class Blogger(BlogBase):
    """
    Blogger posts
    """

    def _regex_id(self):
        try:
            blogname = re.match(r'(http://)?([\w_-]+)(.blogspot.com)?(.*)', self.url).group(2)
        except AttributeError:
            raise goscale_models.WrongAttribute(attribute='url')
        return blogname

    def _get_data_source_url(self):
        # blog has the following possibile aspects: [blogname], [blogname.blogspot.com], [http://blogname.blogspot.com/]
        url = 'http://%s.blogspot.com/feeds/posts/default' % self._regex_id()
        if self.label:
            # append labels to url to get http://[blogname].blogspot.com/feeds/posts/default/-/[label]
            url = '%(url)s/-/%(label)s' % {'url': url, 'label': self.label}
        return url.replace(' ', '%20')

    def _get_title(self, entry):
        title = entry.get('title')
        if title.lower() == 'no title':
            return None
        return title

Blogger._meta.get_field('url').help_text = 'ex: http://blogger-cms.blogspot.com/<br/>ex: blogger-cms'
Blogger._meta.get_field('url').verbose_name = _('Blog Name/URL')
Blogger._meta.get_field('label').help_text = 'ex: Features'

signals.post_save.connect(goscale_models.update_posts, sender=Blogger)


class Tumblr(BlogBase):
    """
    Tumblr posts
    """

    def _regex_id(self):
        try:
            blogname = re.match(r'(http://)?([\w_-]+)(.tumblr.com)?(.*)', self.url).group(2)
        except AttributeError:
            raise goscale_models.WrongAttribute(attribute='url')
        return blogname

    def _get_data_source_url(self):
        # blog has the following possibile aspects: [blogname], [blogname.blogspot.com], [http://blogname.blogspot.com/]
        url = 'http://%s.tumblr.com' % self._regex_id()
        if self.label:
            # append labels to url to get http://[blogname].blogspot.com/feeds/posts/default/-/[label]
            url = '%(url)s/tagged/%(label)s' % {'url': url, 'label': self.label}
        return '%s/rss' % url

Tumblr._meta.get_field('url').help_text = 'ex: http://staff.tumblr.com/<br/>ex: staff'
Tumblr._meta.get_field('url').verbose_name = _('Blog Name/URL')
Tumblr._meta.get_field('label').help_text = 'ex: features'

signals.post_save.connect(goscale_models.update_posts, sender=Tumblr)


import re
import simplejson
import datetime
import urllib2

from BeautifulSoup import BeautifulSoup
from goscale import models as goscale_models
from goscale import utils
from goscale import conf
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext as _


class Form(goscale_models.GoscaleCMSPlugin):
    """
    Google Form
    """
    url = models.URLField(max_length=250, verbose_name=_('Google Form URL'),
        help_text='ex: https://docs.google.com/spreadsheet/viewform?formkey=cDZ5QkRvZDg5d2Z1Y0l0anEyUVNuZEE6MA')

    def _regex_id(self):
        try:
            return self.url.split('key=')[1]
        except:
            raise goscale_models.WrongAttribute(attribute='url')

    def _get_entry_link(self, entry):
        return 'https://docs.google.com/spreadsheet/formResponse?formkey=%s' % self._regex_id()

    def _get_data(self):
        if not self.url:
            return []
        res = urllib2.urlopen(self.url)
        #TBD get hash and compare to cache value
        return [res.read()]

    def _store_post(self, stored_entry, entry):
        # parse form html
        soup = BeautifulSoup(entry)
        form = soup.find('form')
        description = form.renderContents()
#        print description
        # fill in the fields
        stored_entry.content_type = 'text/html'
        stored_entry.link = self._get_entry_link(entry)
        stored_entry.title = 'Form %d' % self.id
        stored_entry.description = description
        return super(Form, self)._store_post(stored_entry)

#signals.post_save.connect(goscale_models.update_posts, sender=Form)



from django.contrib import admin, messages
from models import Theme
from django.conf import settings
from django.utils.translation import ugettext, ugettext_lazy as _
from django.db import models
from django import forms

class ThemeAdminForm(forms.ModelForm):
    class Meta:
        model = Theme

    def clean(self):
        cleaned_data = super(ThemeAdminForm, self).clean()
        name = cleaned_data.get("name")
        theme_file = cleaned_data.get("theme_file")
        if not name and not theme_file:
            # at least one of the fields have to be present
            raise forms.ValidationError(_('Choose at lease one of "Theme file" or "Name".'))
        return cleaned_data

class ThemeAdmin(admin.ModelAdmin):
    list_display = ('id','name',)
    list_display_links = ('id','name',)
    list_filter = ('name',)
    filter_horizontal = ('sites',)
    form = ThemeAdminForm
#    readonly_fields = ('name',)
admin.site.register(Theme, ThemeAdmin)

from cms.models import Page
from cms.admin.pageadmin import PageAdmin
from cms.admin.forms import PageAddForm,PageForm
admin.site.unregister(Page)
t = Page._meta.get_field_by_name("template")[0]
template_choices = [(x, _(y)) for x,y in settings.CMS_TEMPLATES]
t.choices.extend(template_choices)
admin.site.register(Page, PageAdmin)





# -*- coding: utf-8 -*-

import datetime
import hashlib

from django.utils import simplejson

from cms.models.pluginmodel import CMSPlugin
from django.db import models
from django.utils.translation import ugettext as _
from django.core.cache import cache

import unidecode

from django.template import defaultfilters
from goscale import conf, utils


class Post(models.Model):
    link = models.URLField(blank=True, null=True, db_index=True) # link is also unique identifier for each entry
    permalink = models.CharField(max_length=250, blank=True, null=True, db_index=True) # page independent permalink
    content_type = models.CharField(max_length=250, blank=True, null=True, db_index=True) # MIME type for data field
    slug = models.CharField(max_length=250, blank=True, null=True, db_index=True) #short title, text without HTML
    updated = models.DateTimeField(blank=True, null=True, db_index=True)  # last updated
    published = models.DateTimeField(blank=True, null=True, db_index=True) #pubDate
    title = models.CharField(max_length=250, blank=True, null=True) #short title, text without HTML
    description = models.TextField(blank=True, null=True) #universal text/html representation of entry
    author = models.CharField(max_length=250, blank=True, null=True)
    categories = models.CharField(max_length=250, blank=True, null=True, db_index=True)
    summary = models.TextField(blank=True, null=True) #short plain text description
    attributes = models.TextField(blank=True, null=True) #JSON

    def save(self, **kw):
        self.set_slug()  # handle slug
        self.set_links() # handle links
        self.format_categories()
        super(Post, self).save(**kw) # handle links

    def set_slug(self, slug=None):
        if slug:
            self.slug = slug
            return
        if self.title:
            self.slug = defaultfilters.slugify(unidecode.unidecode(self.title))
        else:
            self.slug = str(self.id)
        return

    def set_links(self, permalink=None):
        if permalink:
            self.permalink = permalink
        else:
        #            self.permalink = '/posts/%s/' % self.slug
            self.permalink = self.link
        return

    def format_categories(self):
        if self.categories:
            self.categories = ','.join(['[%s]' % tag.strip(' ') for tag in self.categories.split(',')])
        return

    def dict(self):
        """ Returns dictionary of post fields and attributes
        """
        post_dict = {
            'id': self.id,
            'link': self.link,
            'permalink': self.permalink,
            'content_type': self.content_type,
            'slug': self.slug,
            'updated': self.updated, #.strftime(conf.GOSCALE_ATOM_DATETIME_FORMAT),
            'published': self.published, #.strftime(conf.GOSCALE_ATOM_DATETIME_FORMAT),
            'title': self.title,
            'description': self.description,
            'author': self.author,
            'categories': self.categories[1:-1].split(',') if self.categories else None,
            'summary': self.summary,
            }
        if self.attributes:
            attributes = simplejson.loads(self.attributes)
            post_dict.update(attributes)
        return post_dict

    def json(self, dict=None, indent=None):
        """ Returns post JSON representation
        """
        if not dict:
            dict = self.dict()
        for key, value in dict.iteritems():
            if type(value) == datetime.datetime:
                dict[key] = value.strftime(conf.GOSCALE_ATOM_DATETIME_FORMAT)
        return simplejson.dumps(dict, indent=indent)

    def __unicode__(self):
        return 'post object: %s' % self.title or self.id


class GoscaleCMSPlugin(CMSPlugin):
    """
    Common base abstract class for all the GoScale plugins
    """
    #    plugin_templates = PLUGIN_TEMPLATES
    # Fields
    posts = models.ManyToManyField(Post, verbose_name=_('Posts'))
    template = models.CharField(max_length=255, null=True, blank=True, verbose_name=_('Template'))
    title = models.CharField(max_length=255, null=True, blank=True, verbose_name=_('Title'))
    updated = models.DateTimeField(blank=True, null=True, db_index=True, verbose_name=_('Last updated'))

    # Private Attributes
    _dummy_datetime = utils.get_datetime_now()
    _fields = []

    class Meta:
        abstract = True

    def __unicode__(self):
        template = self.template.replace('.html', '').capitalize() if self.template else None
        title = self.title or template or str(self.id)
        if len(title) > 25:
            title = '%s...' % title[:25]
        return title

    # Override metods
    def copy_relations(self, oldinstance):
        self.posts = oldinstance.posts.all()

    # Public methods
    @classmethod
    def get_fields_list(cls):
        fields_list = []
        ignore_fields = ['changed_date', 'cmsplugin', 'cmsplugin_ptr', 'creation_date', 'id', 'language', 'level',
                         'lft', 'parent', 'placeholder', 'plugin_type', 'position', 'posts', 'rght', 'tree_id']
        for field in cls._meta.get_all_field_names():
            if field not in ignore_fields:
                fields_list.append(field)
        return fields_list

    def get_fields_dict(self):
        fields_dict = {}
        for field in self.__class__.get_fields_list():
            value = self.__getattribute__(field)
            if field == 'updated':
                value = str(value)
            fields_dict[field] = value
        return fields_dict

    def get_cache_key(self, offset=0, limit=0, order=None, post_slug=''):
        """ The return of Get
        """
        return hashlib.sha1(
            '.'.join([
                str(self._get_data_source_url()),
                str(offset),
                str(limit),
                str(order),
                str(post_slug),
                ])
        ).hexdigest()

    def get_posts(self, offset=0, limit=1000, order=None, filters=None):
        """ This method returns list of Posts for this Data Source starting at a given offset and not more than limit
        It will call content-specific methods:
             _format() to format output from the DataStore
        """
        order = self._get_order(order)
        cache_key = self.get_cache_key(offset, limit, order, filters)
        content = cache.get(cache_key)
#        if content:
#            return content
        try:
            if self.up_to_date():
                pass # not time to update yet
            else:
                self.update()
        except:
            raise
            pass # query the database for now and update later
        query = self._get_query(order=order, filters=filters)
        posts = query[int(offset):int(offset)+int(limit)]
        posts = self._format(posts)
        cache_duration = conf.GOSCALE_CACHE_DURATION if posts else 1
        cache.set(cache_key, posts, cache_duration)
        return posts

    def get_post(self, slug):
        """ This method returns a single post by slug
        """
        cache_key = self.get_cache_key(post_slug=slug)
        content = cache.get(cache_key)
        if not content:
            post = Post.objects.get(slug=slug)
            content = self._format(post)
            cache_duration = conf.GOSCALE_CACHE_DURATION if post else 1
            cache.set(cache_key, content, cache_duration)
        return content

    def up_to_date(self):
        """ Returns True if plugin posts are up to date

        Determined by self.updated and conf.GOSCALE_POSTS_UPDATE_FREQUENCY
        """
#        return False
        if not self.updated:
            return False
        return (utils.get_datetime_now() - self.updated).seconds < conf.GOSCALE_POSTS_UPDATE_FREQUENCY

    def update(self):
        """This method should be called to update associated Posts
        It will call content-specific methods:
             _get_data() to obtain list of entries
             _store_post() to store obtained entry object
             _get_data_source_url() to get an URL to identify Posts from this Data Source
        """
        #get the raw data
#        self.posts.all().delete() # TODO: handle in update_posts if source changes without deleting every time
        data = self._get_data()
        #iterate through them and for each item
        msg = []
        for entry in data:
            link = self._get_entry_link(entry)
            stored_entry, is_new = Post.objects.get_or_create(link=link)
            self._store_post(stored_entry, entry)
            if is_new is True:
            #self._set_dates(stored_entry)
            #                self._store_post(stored_entry, entry)
                msg.append('Post "%s" added.' % link)
            else:
                msg.append('Post "%s" already saved.' % link)
        self.updated = utils.get_datetime_now()
        self.save(no_signals=True)
        return '<br />'.join(msg)

    # Private methods
    def _get_data_source_url(self):
        """This method should return URL to identify Posts.
        Different content types may store the URL in a different parameters or even create a dummy one
        """
        if 'url' not in self.__dict__:
            return None
        return self.__getattribute__('url')

    def _get_entry_link(self, entry):
        """ Returns a unique link for an entry

        Most likely an external link from the original source.
        Override it if you need to alter an original link or generate your own.
        """
        return entry.link

    def _get_dummy_datetime(self):
        """ This method is to store posts which have no publish date.
        Please note that post retrieved later will have an earlier publication date - just as it is in RSS feeds.
        """
#        self._dummy_datetime -= datetime.datetime.resolution
        return self._dummy_datetime

    def _get_data(self):
        """ Gets raw data from data source. Returns list of dictionaries
        basically, request a http  or google data
        check hash of response and return [] if feed not changed
        cash response hash for further comparison
        split to entries and return array """
        return []

    def _get_order(self, order=None):
        if order:
            return order
        try:
            return self.__getattribute__('order')
        except AttributeError:
            return conf.GOSCALE_DEFAULT_CONTENT_ORDER

    def _get_query(self, order=None, filters=None):
        """ This method is just to evade code duplication in count() and get_content, since they do basically the same thing"""
        order = self._get_order(order)
        return self.posts.all().order_by(order)

    def _format(self, posts):
        """ This method is called by get_content() method"""
        if posts.__class__ == Post:
            # format a single post
            return posts.dict()
        formated_posts = []
        for post in posts:
            formated_posts.append(post.dict())
        return formated_posts

    def _store_post(self, stored_entry, entry=None):
        """ This method formats entry returned by _get_data() and puts to DB
        create textDesc, title, and MIME """
        #        stored_entry.content_type = utils.get_source_setting(ds.type, 'type')
        if stored_entry.published is None:
            stored_entry.published = self._get_dummy_datetime()
        if stored_entry.updated is None:
            stored_entry.updated = self._get_dummy_datetime()
        #        print 'Post: %s' % stored_entry.title
        #        print 'Attributes: %s' % stored_entry.attributes
        stored_entry.save()
        self.posts.add(stored_entry)
        return None


def update_posts(**kwargs):
    pass # TODO: find a better place to update posts


class WrongAttribute(Exception):
    def __init__(self, message=None, attribute=None):
        if not message:
            if attribute is not None:
                message = 'Attribute "%s" has wrong format' % attribute
            else:
                message = 'Attribute has wrong format'
            # Call the base class constructor with the parameters it needs
        Exception.__init__(self, message)


import urllib2
import urllib

from django import http
from django.utils.translation import ugettext as _


def form(request):
    """ Ajax handler for Google Form submition
    """
    if request.method == 'POST':
        submit_url = '%s&hl=%s' % (
            request.POST['url'],
            request.LANGUAGE_CODE
        )
        params = urllib.urlencode(request.POST)
        f = urllib2.urlopen(submit_url, params)
        text = f.read()
    else:
        text = _('Error: request type has to be POST')

    response = http.HttpResponse(text, mimetype="text/plain")
    return response



import re
import feedparser
import urllib2
import json as simplejson
import datetime
from goscale import models as goscale_models
from goscale import utils
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext as _


class VideosBase(goscale_models.GoscaleCMSPlugin):
    """
    YouKu Videos
    """
    playlist = models.CharField(max_length=250, null=True, blank=True, verbose_name=_('Playlist'),
        help_text='Playlist URL')
    lightbox = models.BooleanField(default=False, verbose_name=_('Open videos in a lightbox'),
        help_text=_('If checked videos will open in a lightbox, otherwise inline.'))

    class Meta:
        abstract = True


class YouKu(VideosBase):
    """
    YouKu Videos
    """
    def _regex_id(self, link=None):
        link = link or self.playlist
        try:
            # this regex can also handle unicode encoded characters in the id.
            return re.match(r'(http://)?(u.|www.)?(youku.com/)?(playlist_show/id_|user_show/uid_)?((.+)[^\.html])', link).group(5)
        except AttributeError:
            raise goscale_models.WrongAttribute(attribute='playlist')

    def _get_data_source_url(self):
        if self.playlist:
            id = self._regex_id(self.playlist)
            return 'http://www.youku.com/playlist_show/id_%s' % id
        return ''

    def _get_data(self):
        try:
            if self.playlist:
                id = self._regex_id(self.playlist) # http://www.youku.com/playlist/rss/id/ + id
                feed_url = 'http://www.youku.com/playlist/rss/id/%s' % id
                req = utils.forge_request(feed_url.encode('utf-8'))
                rss = urllib2.urlopen(req).read()
                feed = feedparser.parse(rss)
                data = []
                for entry in feed.entries:
                    data.append(utils.dict2obj(entry))
                return data
        except:
            pass
        return []

    def _store_post(self, stored_entry, entry):
        video_id = entry.link.replace('http://www.youku.com/v_show/id_', '').split('_')[0]
        arr = entry.summary.split(' src="')
        if len(arr) == 1:
            thumbnail = None
        else:
            thumbnail = arr[1].split('"')[0]
        try:
            datetime_obj = datetime.datetime.strptime(entry.updated[0:-6], "%a, %d %b %Y %H:%M:%S") # Sun, 26 Apr 2009 00:45:30
            stored_entry.published = datetime_obj - datetime.timedelta(minutes=480) #convert to GMT
        except:
            stored_entry.published = None
        stored_entry.title = entry.title
        stored_entry.author = entry.author
        stored_entry.category = entry.itunes_keywords
        stored_entry.summary = entry.summary
        video_url = ''
        for link in entry.links:
            if link.rel == 'enclosure':
                video_url = link.href
        stored_entry.attributes = simplejson.dumps({
            'video_id': video_id,
            'thumbnail': thumbnail,
            'video_url': video_url,
            })
        return super(YouKu, self)._store_post(stored_entry)

YouKu._meta.get_field('playlist').help_text = 'ex: http://www.youku.com/playlist_show/id_17031624.html'
YouKu._meta.get_field('playlist').verbose_name = _('Youku playlist')

signals.post_save.connect(goscale_models.update_posts, sender=YouKu)


class Youtube(VideosBase):
    """
    Youtube videos
    """
    channel = models.CharField(max_length=250, blank=True, null=True, verbose_name=_('Youtube channel'),
        help_text='ex: https://www.youtube.com/user/GoogleDevelopers')

    def _regex_id(self, url=None):
        id = None
        try:
            url = url or self.playlist
            id = re.search('(list=)([\d\w._]+)', url).group(2)
        except AttributeError:
            try:
                url = url or self.channel
                id = re.search('(\/user\/)([\d\w+._]+)', url).group(2)
            except AttributeError:
                raise goscale_models.WrongAttribute(attribute='channel')
        if not id:
            raise goscale_models.WrongAttribute(attribute='playlist')
        return id

    def _get_data_source_url(self):
        if self.playlist:
            # try youtube_playlist.
            id = self._regex_id(self.playlist)
            url = 'http://www.youtube.com/view_play_list?p=%s' % id
        elif self.channel:
            # youtube_channel
            id = self._regex_id(self.channel)
            url = 'http://www.youtube.com/user/%s' % id
        else:
            url = None
        return url

    def _get_data(self):
        from gdata.youtube import service
        try:
            video_service = service.YouTubeService()
            if self.channel:
                # youtube_channel
                query = service.YouTubeVideoQuery()
                query.author = self._regex_id(self.channel)
                query.orderby = 'published'
                feed = video_service.YouTubeQuery(query)
            elif self.playlist:
                # if not youtube_channel, then it's youtube_playlist.
                playlist = self._regex_id(self.playlist)
                feed = video_service.GetYouTubePlaylistVideoFeed(uri='http://gdata.youtube.com/feeds/api/playlists/%s?orderby=published' % playlist)
            return feed.entry
        except:
            return []

    def _store_post(self, stored_entry, entry):
        video_id = entry.link[0].href.replace('http://www.youtube.com/watch?v=', '').split('&')[0]
        if entry.rating and entry.rating.average:
            entry_rating = int(round(float(entry.rating.average)*10))
            if entry_rating%5 > 2:
                video_rating = entry_rating - (entry_rating%5) + 5
            else:
                video_rating = entry_rating - (entry_rating%5)
        else:
            video_rating = 0

        if entry.statistics and entry.statistics.view_count:
            video_view_count = int(entry.statistics.view_count)
        else:
            video_view_count = 0
        stored_entry.title = unicode(entry.title.text, "utf-8")
        stored_entry.author = unicode(entry.author[0].name.text, "utf-8")
        stored_entry.categories = entry.media.category[0].text
        stored_entry.attributes = simplejson.dumps({
            'id': video_id,
            'author_url': 'http://youtube.com/profile?user=%s' % entry.author[0].name.text,
            'thumbnail': 'http://i.ytimg.com/vi/%s/2.jpg' % video_id,
            'video_url': 'http://www.youtube.com/v/%s&hl=en&fs=1' % video_id,
            'rating': video_rating,
            'view_count': video_view_count
        })
        return super(Youtube, self)._store_post(stored_entry)

Youtube._meta.get_field('playlist').help_text = 'ex: https://www.youtube.com/watch?v=tV7IhJwU3hA&list=PL53B883EBFFAE300C'
Youtube._meta.get_field('playlist').verbose_name = _('YouTube playlist')

signals.post_save.connect(goscale_models.update_posts, sender=Youtube)


class Vimeo(VideosBase):
    """
    Vimeo videos
    """
    user = models.CharField(max_length=250, blank=True, null=True, verbose_name=_('Vimeo user'),
        help_text='ex: http://vimeo.com/amandanedermeijer')

    def _regex_id(self, url=None):
        id = None
        try:
            url = url or self.playlist
            id = re.search('(\/channels\/)([\d\w._]+)', url).group(2)
        except AttributeError:
            try:
                url = url or self.user
                id = re.search('(vimeo.com/)([\d\w+._]+)', url).group(2)
            except AttributeError:
                raise goscale_models.WrongAttribute(attribute='user')
        if not id:
            raise goscale_models.WrongAttribute(attribute='playlist')
        return id

    def _get_data_source_url(self):
        if self.playlist:
            id = self._regex_id(self.playlist)
            url = 'http://vimeo.com/channels/%s' % id # vimeo channel  http://vimeo.com/channels/...
        elif self.user:
            id = self._regex_id(self.user)
            url = 'http://vimeo.com/%s' % id # vimeo user  http://vimeo.com/...
        else:
            url = None
        return url

    def _get_data(self):
        try:
            if self.playlist:
                id = self._regex_id(self.playlist) # http://vimeo.com/channels/iphonehd
                feed_url = 'http://vimeo.com/api/v2/channel/%s/videos.json' % id
            elif self.user:
                id = self._regex_id(self.user) # http://vimeo.com/wyldstallyons
                feed_url = 'http://vimeo.com/api/v2/%s/videos.json' % id
            req = utils.forge_request(feed_url.encode('utf-8'))
            json = urllib2.urlopen(req).read()
            feed = simplejson.loads(json)
            # jsonfeed is a list of dictionaries
            # For update(), we need a list of objects. transforming the feed below.
            data = []
            for entry in feed:
                entry['link'] = entry['url']
                data.append(utils.dict2obj(entry))
            return data
        except:
            raise
            return []

    def _store_post(self, stored_entry, entry):
        stored_entry.title = entry.title
        stored_entry.author = entry.user_name
        stored_entry.categories = entry.tags
        stored_entry.published = entry.upload_date
        stored_entry.attributes = simplejson.dumps({
            'id': entry.id,
            'comments_id': entry.id,
            'description': entry.description,
            'author_url': entry.user_url,
            'thumbnail': entry.thumbnail_medium,
            'video_url':  'http://vimeo.com/moogaloop.swf?clip_id=%s&amp;server=vimeo.com&amp;fullscreen=1&amp;show_title=1&amp;show_byline=1&amp;show_portrait=1&amp;color=00ADEF' % entry.id,
#            'likes_count': entry.stats_number_of_likes,
#            'view_count': entry.stats_number_of_views,
#            'comments_count': entry.stats_number_of_comments,
            })
        return super(Vimeo, self)._store_post(stored_entry)

Vimeo._meta.get_field('playlist').help_text = 'ex: http://vimeo.com/channels/6513'
Vimeo._meta.get_field('playlist').verbose_name = _('Vimeo channel')

signals.post_save.connect(goscale_models.update_posts, sender=Vimeo)


import tarfile
import StringIO
import shutil
import os

from django.conf import settings
from django.db import models
from django.contrib.sites.models import Site
from django.db.models.signals import post_save, pre_save, post_delete, m2m_changed
from django.utils.translation import ugettext, ugettext_lazy as _
from goscale.themes import set_themes

class Theme(models.Model):
    sites = models.ManyToManyField(Site, related_name='themes', null=True, blank=True)
    name = models.CharField(max_length=255, blank=True, help_text=_(
        'Only set name if no theme file provided (should be consistent with directory name).'
    ))
    theme_file = models.FileField(upload_to='themes_archives', null=True, blank=True)
    
    def save(self, *args, **kwargs):
        if not self.id and not self.name:
            f = tarfile.open(fileobj=self.theme_file, mode='r:gz')
            self.name = f.getnames()[-1]
            f.extractall(settings.THEMES_DIR)

        super(Theme, self).save(*args, **kwargs)
                
    
    class Meta:
        verbose_name = "Theme"
        verbose_name_plural = "Themes"

    def __unicode__(self):
        return self.name

def delete_themes(sender, **kwargs):
    instance = kwargs['instance']
    try:
        shutil.rmtree(os.path.join(settings.THEMES_DIR, instance.name))
    except OSError: 
        pass
    set_themes()

def theme_site_m2m_changes(sender, **kwargs):
    instance = kwargs['instance']
    action = kwargs['action']
    if action in ("post_add", "post_remove", "post_clear"):
        if type(instance) is Theme:
            for site in instance.sites.all():
                for theme in site.themes.all():
                    if theme.id != instance.id:
                        site.theme_set.remove(theme)        
        set_themes()
post_delete.connect(delete_themes, sender=Theme)
m2m_changed.connect(theme_site_m2m_changes, sender=Theme.sites.through)



import re
import os
import simplejson
import datetime
import urllib2
from dateutil import parser
from gdata.calendar import client
from goscale import models as goscale_models
from goscale import utils
from goscale import conf
from goscale import decorators
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext as _


class Calendar(goscale_models.GoscaleCMSPlugin):
    """
    Google Calendar Events
    """
    url = models.CharField(max_length=250, verbose_name=_('Calendar feed URL'),
        help_text=_('XML link from "Private Address" in your "Calendar Settings"<br/> \
            ex: https://www.google.com/calendar/feeds/5qc0048gbti1s87faijksu67lc%40group.calendar.google.com/private-31d8c813eaadb4e9e6a3c73961e404e4/basic'))
    page_size = models.PositiveSmallIntegerField(default=conf.GOSCALE_DEFAULT_PAGE_SIZE,
        verbose_name=_('Events per page'), help_text=_('set 0 for unlimited.'))
    show_past = models.BooleanField(default=False, verbose_name=_('Show past events'),
        help_text=_('If set past events will be shown.'))
    order = 'updated'

    def get_past_events(self):
        query = self.posts.all()
        return self._format(query.filter(updated__lt=datetime.datetime.now()).order_by('-updated')[:self.page_size])

    def _get_query(self, order=None, filters=None):
        order = self._get_order(order)
        query = self.posts.all()
        start = datetime.datetime.now()
        if filters and 'start' in filters:
            start = parser.parse(filters['start'])
        query = query.filter(updated__gte=start)
        return query.order_by(order)

    def _get_data(self):
        cal_client = client.CalendarClient()
        feed = cal_client.get_calendar_event_feed(uri=self.url+'?hl=en')
        events = []
        for i, an_event in enumerate(feed.entry):
            events.append(an_event)
        #        events = self._changeEventsTime(events)
        return events

    def _get_entry_link(self, entry):
        return entry.link[0].href

    def _store_post(self, stored_entry, entry):
        location = None
#        a_when = None
#        for a_when in entry.when:
#            for a_where in entry.where:
#                if a_where.value_string is not None:
#                    location = a_where.value_string
#        if entry.content.text != None:
#            stored_entry.description = entry.content.text
        attributes = {
            'event_id': entry.link[0].href.split('eid=')[1],
            'event_id2': entry.id.text,
        }
        for attr in entry.content.text.split('<br />'):
            if ':' not in attr:
                continue
            [key, val] = attr.split(':', 1)
            attributes[key.strip().lower().replace(' ', '_')] = val.strip()
        stored_entry.title = entry.title.text
        stored_entry.link = self._get_entry_link(entry)
        try:
            stored_entry.updated = parser.parse(attributes['when'].split('to')[0]) # parse event start date
        except ValueError:
            raise
            stored_entry.updated = datetime.datetime.strptime(entry.updated.text, '%Y-%m-%dT%H:%M:%S.%fZ')
        stored_entry.published = datetime.datetime.strptime(entry.published.text, '%Y-%m-%dT%H:%M:%S.%fZ')
        stored_entry.author = entry.author[0].name.text
        stored_entry.attributes = simplejson.dumps(attributes)
        stored_entry.description = attributes.get('event_description')
        return super(Calendar, self)._store_post(stored_entry)

    def _parse_datetime(self, when):
        return datetime.datetime.strptime(when, '%Y-%m-%dT%H:%M:%S.%fZ')

signals.post_save.connect(goscale_models.update_posts, sender=Calendar)


# -*- coding: utf-8 -*-
import simplejson
import urllib
from classytags.arguments import Argument, MultiValueArgument, MultiKeywordArgument
from classytags.core import Options
from classytags.helpers import InclusionTag, AsTag
from django import template
from cms.templatetags import cms_tags
from cms.models import CMSPlugin
from sekizai.templatetags import sekizai_tags
from goscale import models
from goscale import cms_plugins

register = template.Library()


class Paginator(InclusionTag):
    name = 'goscale_paginator'
    template = 'paginator.html'
    options = Options(
        MultiKeywordArgument('params', required=False)
    )

    def get_template(self, context, **kwargs):
        """
        Returns the template to be used for the current context and arguments.
        """
        if 'template' in kwargs['params']:
            self.template = kwargs['params']['template']
        return super(Paginator, self).get_template(context, **kwargs)

    def get_context(self, context, params):
        paginator = context['paginator']
        page = context['page']
        page_range = []
        if len(paginator.page_range) > 6:
            page_range.extend(paginator.page_range[:5])
            if page.number > 5:
                if page.number > 7:
                    page_range.append('...')
                if page.number == 6:
                    page_range.extend(paginator.page_range[(page.number-1):(page.number+1)])
                else:
                    page_range.extend(paginator.page_range[(page.number-2):(page.number+1)])
            elif page.number == 5:
                page_range.append(6)
            if len(paginator.page_range) - page.number > 2:
                page_range.append('...')
            if len(paginator.page_range) - page.number > 1:
                page_range.append(paginator.page_range[-1])
        else:
            page_range = paginator.page_range
        context['page_range'] = page_range
        return context

register.tag(Paginator)


class PluginFilters(AsTag):
    name = 'goscale_plugin_filters'
    options = Options(
        MultiKeywordArgument('params', required=False),
        'as',
        Argument('varname', resolve=False, required=False),
    )

    def get_value(self, context, params):
        filters = []
        is_global = params.pop('global') if 'global' in params else ''
        is_exclusive = params.pop('exclusive') if 'exclusive' in params else ''
        if is_global.lower() in ['true', 'yes']:
            plugin_filters = 'plugin_filters'
        else:
            plugin_filters = 'plugin_%s_filters' % context['plugin_id']
        for key, val in params.iteritems():
            filters.append('='.join([key, str(val)]))
        qs = ['='.join([plugin_filters, '|'.join(filters)])]
        if is_exclusive.lower() not in ['true', 'yes']:
            for key, value in context['request'].GET.iteritems():
                if key.startswith('_') or key == plugin_filters:
                    continue
                qs.append('='.join([key, value]))
        return '?' + '&'.join(qs)

register.tag(PluginFilters)


class PluginPost(AsTag):
    name = 'goscale_plugin_post'
    options = Options(
        Argument('post'),
        'as',
        Argument('varname', resolve=False, required=False),
    )

    def get_value(self, context, post):
        return '?post=%s&plugin_id=%s' % (post['slug'], context['plugin_id'])

register.tag(PluginPost)


class GoscalePlaceholder(cms_tags.Placeholder):
    """
    This template node is used to output page content and
    is also used in the admin to dynamically generate input fields.

    eg: {% placeholder "placeholder_name" %}

    {% placeholder "sidebar" inherit %}

    {% placeholder "footer" inherit or %}
        <a href="/about/">About us</a>
    {% endplaceholder %}

    Keyword arguments:
    name -- the name of the placeholder
    width -- additional width attribute (integer) which gets added to the plugin context
    (deprecated, use `{% with 320 as width %}{% placeholder "foo"}{% endwith %}`)
    inherit -- optional argument which if given will result in inheriting
        the content of the placeholder with the same name on parent pages
    or -- optional argument which if given will make the template tag a block
        tag whose content is shown if the placeholder is empty
    """
    name = 'goscale_placeholder'
    options = cms_tags.PlaceholderOptions(
        Argument('name', resolve=False),
        MultiKeywordArgument('goscale', required=False),
        MultiValueArgument('extra_bits', required=False, resolve=False),
        blocks=[
            ('endplaceholder', 'nodelist'),
        ]
    )

    def render_tag(self, context, *args, **kwargs):
        if 'goscale' in kwargs:
            if 'render_posts' in kwargs['goscale']:
                if kwargs['goscale']['render_posts'].lower() == 'true':
                    if 'request' in context:
                        request = context['request']
                        slug = request.GET.get('post')
                        plugin_id = request.GET.get('plugin_id')
                        if slug:
                            if plugin_id:
                                plugin = CMSPlugin.objects.get(pk=plugin_id)
                                instance = plugin.get_plugin_instance()[0]
                                plugin = plugin.get_plugin_class()
                            else:
                                instance = None
                                plugin = cms_plugins.GoscaleCMSPluginBase
                            return plugin.render_post(slug=slug, instance=instance)
#                            post = models.Post.objects.get(slug=slug)
#                            raise ValueError
#                            return post.json()
            if 'ajaxify' in kwargs['goscale']:
                pass # TODO: Ajaxify placeholders
#            return kwargs
            kwargs.pop('goscale')
        return super(GoscalePlaceholder, self).render_tag(context, *args, **kwargs)

register.tag(GoscalePlaceholder)


class GoscaleAddtoBlock(sekizai_tags.Addtoblock):
    name = 'goscale_addtoblock'

    def render_tag(self, *args, **kwargs):
        context = args[0]
        request = context.get('request')
        if request and (request.is_ajax() or 'ajax' in request.GET):
            nodelist = kwargs['nodelist']
            return nodelist.render(context)
        return super(GoscaleAddtoBlock, self).render_tag(*args, **kwargs)

register.tag(GoscaleAddtoBlock)